import 'dart:async';
import 'package:flutter/services.dart';
import 'linkv_api_defines.dart';
import 'linkv_video_callback.dart';

class LinkVRTCEngine {
  static const MethodChannel _channel = const MethodChannel('LVRTCEngine');
  static LinkVVideoCallback _videoCallback;

  static Future<String> get platformVersion async {
    final String version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  static Future<String> getSdkVersion() async {
    final String version = await _channel.invokeMethod("getSdkVersion");
    return version;
  }

  static Future<int> initSDK(String appid, String appSign, bool isTestEnv, bool isInternationalEnv) async {
    final int result = await _channel.invokeMethod("initSDK", {
      'AppId': appid,
      'AppSign': appSign,
      'isTestEnv': isTestEnv,
      'isInternationalEnv':isInternationalEnv
    });
    return result;
  }

  static void unInitSDK() async {
    await _channel.invokeMethod("unInitSDK");
  }

  static void setVideoCallback(LinkVVideoCallback callback) {
    _videoCallback = callback;
    _channel.setMethodCallHandler(_handleVideoCallback);
  }

  static Future<LinkvLoginRoomResult> loginRoom(String roomId, String userId, bool isHost, bool isAudioOnly) async {
    final Map<dynamic, dynamic> resultMap = await _channel.invokeMethod("loginRoom", {
      "roomId":roomId,
      "userId":userId,
      "isHost":isHost,
      "isAudioOnly":isAudioOnly
    });
    LinkvLoginRoomResult result = LinkvLoginRoomResult();
    List<LVUser> users = List<LVUser>();
    List<dynamic> nativeList = resultMap["users"];
    if(nativeList != null) {
      for(Map<dynamic,dynamic> item in nativeList) {
        LVUser user = LVUser();
        user.userId = item["userId"];
        List<String> pullUrls = List<String>();
        if(item["pullUrls"] != null) {
          for(var url in item["pullUrls"]) {
            pullUrls.add(url);
          }
        }
        user.pullUrls = pullUrls;
        users.add(user);
      }
    }
    result.code = resultMap["code"];
    result.users = users;
    return result;
  }

  static void setPublishQualityMonitorCycle(int seconds) async {
    await _channel.invokeMethod("setPublishQualityMonitorCycle", {'seconds':seconds});
  }

  static void startPublishing() async {
    await _channel.invokeMethod("startPublishing");
  }

  static void setPlayQualityMonitor(int seconds) async {

    await _channel.invokeMethod("setPlayQualityMonitor", {"seconds": seconds});
  }

  static void startPlayingStream(String streamId) async {
    if (streamId == null || streamId.length == 0) return;

    await _channel.invokeMethod("startPlayingStream", {"streamId": streamId});
  }

  static void stopPlayingStream(String streamId) async {
    if (streamId == null || streamId.length == 0) return;

    await _channel.invokeMethod("stopPlayingStream", {"streamId": streamId});
  }

  static void logoutRoom() async {
    await _channel.invokeMethod("logoutRoom");
  }

  static void stopCapture() async {
    await _channel.invokeMethod("stopCapture");
  }

  static void startCapture() async {
    await _channel.invokeMethod("startCapture");
  }

  static void stopPublishStream() async {
    await _channel.invokeMethod("stopPublishStream");
  }

  static void switchCamera(LVCameraPosition position) async {
    await _channel.invokeMethod("switchCamera", {"cameraPosition": position.index});
  }

  static void setAVConfig(LVAVConfig config) async {
    await _channel.invokeMethod("setAVConfig", {"config": config.index});
  }

  // MARK: - Video Callback
  static Future<dynamic> _handleVideoCallback(MethodCall call) {
//    print("D/Wing LinkVRTCEngine _handleVideoCallback method = " + call.method);
    if (call.method == "onDrawFrame") {
      _videoCallback?.onDrawFrame(0, 0, 0, "s", "s1");
    } else if (call.method == "onAddRemoter") {
      LVUser user = LVUser();
      user.userId = call.arguments["userId"];
      if(call.arguments["pullUrls"] != null) {
        List<String> pullUrls = List<String>();
        for(var url in call.arguments["pullUrls"]) {
          pullUrls.add(url);
        }
        user.pullUrls = pullUrls;
      }
      print("D/Wing LinkVRTCEngine onAddRemoter = " + user.userId);
      _videoCallback?.onAddRemoter(user);
    } else if (call.method == "onDeleteRemoter") {
        _videoCallback?.onDeleteRemoter(call.arguments["streamId"]);
    } else if (call.method == "onKickOff") {
      _videoCallback?.onKickOff(call.arguments["state"] ,call.arguments["streamId"]);
    } else if (call.method == "onRoomDisconnect") {
      _videoCallback?.onRoomDisconnect(call.arguments["state"]);
    } else if (call.method == "onRoomReconnect") {
      _videoCallback?.onRoomReconnect();
    } else if (call.method == "onMixComplete") {
      _videoCallback?.onMixComplete(call.arguments["b"]);
    } else if (call.method == "onPublishQualityUpdate") {
      LVVideoStatistic videoStatistic = LVVideoStatistic.convertFromMap(call.arguments);
      _videoCallback?.onPublishQualityUpdate(videoStatistic);
    } else if (call.method == "onPublishStateUpdate") {
      _videoCallback?.onPublishStateUpdate(call.arguments["state"]);
    } else if (call.method == "onPlayQualityUpdate") {
      LVVideoStatistic videoStatistic = LVVideoStatistic.convertFromMap(call.arguments);
      _videoCallback?.onPlayQualityUpdate(videoStatistic, call.arguments["uid"]);
    } else if (call.method == "onPlayStateUpdate") {
      _videoCallback?.onPlayStateUpdate(call.arguments["state"], call.arguments["streamId"]);
    } else if (call.method == "onExitRoomComplete") {
      _videoCallback?.onExitRoomComplete();
    }

    return Future.value(true);
  }

}
